"""Module defining Turbomole calculation types."""

from emmet.core.utils import ValueEnum


class RunType(ValueEnum):
    """Turbomole calculation run types."""


class TaskType(ValueEnum):
    """Turbomole calculation task types."""


class CalcType(ValueEnum):
    """Turbomole calculation types."""
